#ifndef NO_FFTW
#include "main.h"
#include "DFT.h"
#include "Geometry.h"
#include "Stack.h"
#include <fftw3.h>

void DCT::help() {
    printf("\n-dct performs a real discrete cosine transform on the current image.\n\n"
	   "Usage: ImageStack -load a.tga -dct -save freq.tga\n\n");
    
}

void DCT::parse(vector<string> args) {
    assert(args.size() == 0, "-dct takes no arguments\n");
    Image im = apply(stack(0));
    pop();
    push(im);
}

Image DCT::apply(Window im) {
    // convert the input to doubles
    double *data = (double *)fftw_malloc(sizeof(double)* im.width * im.height * im.frames);
    fftw_plan p; 


    Image out(im.frames, im.width, im.height, im.channels);
    for (int c = 0; c < im.channels; c++) {
	// read in the data
	double *dataPtr = data;
	for (int t = 0; t < im.frames; t++) {
	    for (int y = 0; y < im.height; y++) {
		for (int x = 0; x < im.width; x++) {
		    *dataPtr++ = im(t, x, y)[c];
		}
	    }
	}
        
        if (im.height == 1 && im.frames == 1 || im.frames == 1 && im.width == 1 || im.width == 1 && im.height == 1) {
            int size = 0;
            if (im.frames > 1) size = im.frames;
            else if (im.height > 1) size = im.height;
            else if (im.width > 1) size = im.width;
            p = fftw_plan_r2r_1d(size, data, data, FFTW_REDFT00, FFTW_ESTIMATE);
        } else if (im.frames == 1 || im.width == 1 || im.height == 1) {
            int w = 0, h = 0;
            if (im.frames == 1) {
                h = im.height;
                w = im.width;
            } else if (im.height == 1) {
                h = im.frames;
                w = im.width;
            } else if (im.width == 1) {
                h = im.frames;
                w = im.height;
            }

	    p = fftw_plan_r2r_2d(h, w, data, data,
				 FFTW_REDFT00, FFTW_REDFT00, FFTW_ESTIMATE);
	    assert(p, "fftw planning failed\n");
	} else {
	    p = fftw_plan_r2r_3d(im.frames, im.height, im.width, data, data,
				 FFTW_REDFT00, FFTW_REDFT00, FFTW_REDFT00, FFTW_ESTIMATE);
	    assert(p, "fftw planning failed\n");
	}

	fftw_execute(p);

	// read out the data
	dataPtr = data;
	float mult = 0.5f / sqrt((float)(im.width * im.height * im.frames));

	for (int t = 0; t < out.frames; t++) {
	    for (int y = 0; y < out.height; y++) {
		for (int x = 0; x < out.width; x++) {
		    out(t, x, y)[c] = (float)(*dataPtr++ * mult);
		}
	    }
	}
	
	fftw_destroy_plan(p);
    }

    fftw_free(data);

    return out;
}

#endif
