#ifndef FILTER_H
#define FILTER_H

class GaussianBlur : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, float filterFrames, float filterWidth, float filterHeight);
};

class FastBlur : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static void apply(Window im, float filterFrames, float filterWidth, float filterHeight, bool addMargin = true);
  private:
    // IIR filters
    static void blurX(Window im, float filterWidth, int tapSpacing = 1);
    static void blurY(Window im, float filterHeight, int tapSpacing = 1);
    static void blurT(Window im, float filterFrames, int tapSpacing = 1);

    // helper function for IIR filtering
    static void calculateCoefficients(float sigma, float *c0, float *c1, float *c2, float *c3);
};

class ChromaBlur : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, float spatialSigma, float colorSigma);
};

class LanczosBlur : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, float filterFrames, float filterWidth, float filterHeight);
};

class Bilateral : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, float spatialSigma, float colorSigma);
};

class JointBilateral : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im1, Window reference, float spatialSigma, float colorSigma);
};

class BilateralSharpen : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, float spatialSigma, float colorSigma, float sharpness);
};

class MedianFilter : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, int radius);
};

class PercentileFilter : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, int radius, float percentile);
};

class CircularFilter : public Operation {
  public:
    void help();
    void parse(vector<string> args);
    static Image apply(Window im, int radius);
};

#endif
