#include "main.h"
#include "Operation.h"

// Include your operation's source file or header here.
// It should must define a subclass of Operation (in Operation.h).

// You might want to look in Arithmetic.cpp/Arithmetic.h for some examples.

// Also check main.h for functions available to parse input, handle errors, etc.

#include "Help.h"
#include "Control.h"
#include "Statistics.h"
#include "Arithmetic.h"
#include "Alignment.h"
#include "Stack.h"
#include "Geometry.h"
#include "Convolve.h"
#include "Color.h"
#include "Paint.h"
#include "File.h"
#include "Calculus.h"
#include "Display.h"
#include "NetworkOps.h"
#include "Panorama.h"
#include "DFT.h"
#include "Wavelet.h"
#include "Filter.h"
#include "HDR.h"


// add your operation to the operations table here
void loadOperations() {
    operationMap["-help"] = new Help();

    // program control
    operationMap["-loop"] = new Loop();
    operationMap["-pause"] = new Pause();
    operationMap["-time"] = new Time();

    // statistics

    operationMap["-statistics"] = new Statistics();
    operationMap["-dimensionreduction"] = new DimensionReduction();
    operationMap["-dimensions"] = new Dimensions();
    operationMap["-noise"] = new Noise();
    operationMap["-histogram"] = new Histogram();
    operationMap["-equalize"] = new Equalize();
    operationMap["-shuffle"] = new Shuffle();
    operationMap["-kmeans"] = new KMeans();
    operationMap["-sort"] = new Sort();
    operationMap["-localmaxima"] = new LocalMaxima();
    operationMap["-printf"] = new Printf();
    operationMap["-fprintf"] = new FPrintf();
    operationMap["-pca"] = new PCA();
    operationMap["-patchpca"] = new PatchPCA();
    operationMap["-rms"] = new RMS();

    // file IO
    operationMap["-load"] = new Load();
    operationMap["-save"] = new Save();
    operationMap["-loadframes"] = new LoadFrames();
    operationMap["-saveframes"] = new SaveFrames();
    operationMap["-loadarray"] = new LoadArray();
    operationMap["-savearray"] = new SaveArray();
    operationMap["-loadblock"] = new LoadBlock();
    operationMap["-saveblock"] = new SaveBlock();
    operationMap["-createtmp"] = new CreateTmp();

    // basic math
    operationMap["-add"] = new Add();
    operationMap["-multiply"] = new Multiply();
    operationMap["-subtract"] = new Subtract();
    operationMap["-divide"] = new Divide();
    operationMap["-max"] = new Maximum();
    operationMap["-min"] = new Minimum();
    operationMap["-log"] = new Log();
    operationMap["-exp"] = new Exp();
    operationMap["-offset"] = new Offset();
    operationMap["-scale"] = new Scale();
    operationMap["-gamma"] = new Gamma();
    operationMap["-mod"] = new Mod();
    operationMap["-normalize"] = new Normalize();
    operationMap["-clamp"] = new Clamp();
    operationMap["-denan"] = new DeNaN();
    operationMap["-threshold"] = new Threshold();
    operationMap["-abs"] = new Abs();
    operationMap["-quantize"] = new Quantize();

    // stack ops
    operationMap["-pop"] = new Pop();
    operationMap["-push"] = new Push();
    operationMap["-pull"] = new Pull();
    operationMap["-dup"] = new Dup();

    // geometry
    operationMap["-resample"] = new Resample();
    operationMap["-crop"] = new Crop();
    operationMap["-flip"] = new Flip();
    operationMap["-adjoin"] = new Adjoin();
    operationMap["-transpose"] = new Transpose();
    operationMap["-translate"] = new Translate();
    operationMap["-paste"] = new Paste();
    operationMap["-downsample"] = new Downsample();
    operationMap["-upsample"] = new Upsample();
    operationMap["-rotate"] = new Rotate();
    operationMap["-affinewarp"] = new AffineWarp();
    operationMap["-expand"] = new Expand();
    operationMap["-tile"] = new Tile();
    operationMap["-subsample"] = new Subsample();
    operationMap["-warp"] = new Warp();
    operationMap["-riffle"] = new Riffle();
    operationMap["-unriffle"] = new Unriffle();
    operationMap["-tileframes"] = new TileFrames();
    operationMap["-frametiles"] = new FrameTiles();
    operationMap["-reshape"] = new Reshape();

    // alignment
    operationMap["-align"] = new Align();

    // color stuff
    operationMap["-colormatrix"] = new ColorMatrix();
    operationMap["-colorconvert"] = new ColorConvert();
    operationMap["-demosaic"] = new Demosaic();

    // convolutions
    operationMap["-convolve"] = new Convolve();
    operationMap["-deconvolve"] = new Deconvolve();
    operationMap["-deconvolvecg"] = new DeconvolveCG();

    // calculus
    operationMap["-gradient"] = new Gradient();
    operationMap["-integrate"] = new Integrate();
    operationMap["-laplacian"] = new Laplacian();
    operationMap["-poisson"] = new Poisson();

    #ifndef NO_FFTW
    // discrete fourier transforms
    operationMap["-dct"] = new DCT();
    #endif

    // painting stuff
    operationMap["-eval"] = new Eval();
    operationMap["-evalchannels"] = new EvalChannels();
    operationMap["-plot"] = new Plot();
    operationMap["-composite"] = new Composite();

    // the display code
    operationMap["-display"] = new Display();

    // network stuff
    operationMap["-send"] = new Send();
    operationMap["-receive"] = new Receive();

    // Panorama stuff
    operationMap["-loadpanorama"] = new LoadPanorama();
    operationMap["-panoramabackground"] = new PanoramaBackground();

    // wavelet stuff
    operationMap["-haar"] = new Haar();
    operationMap["-inversehaar"] = new InverseHaar();
    operationMap["-daubechies"] = new Daubechies();
    operationMap["-inversedaubechies"] = new InverseDaubechies();

    // some filters
    operationMap["-gaussianblur"] = new GaussianBlur();
    operationMap["-fastblur"] = new FastBlur();
    operationMap["-lanczosblur"] = new LanczosBlur();
    operationMap["-circularfilter"] = new CircularFilter();
    operationMap["-bilateral"] = new Bilateral();
    operationMap["-jointbilateral"] = new JointBilateral();
    operationMap["-bilateralsharpen"] = new BilateralSharpen();
    operationMap["-chromablur"] = new ChromaBlur();
    operationMap["-medianfilter"] = new MedianFilter();
    operationMap["-percentilefilter"] = new PercentileFilter();

    // HDR stuff
    operationMap["-assemblehdr"] = new AssembleHDR();

}


void unloadOperations() {
    OperationMapIterator i;
    for (i = operationMap.begin(); i != operationMap.end(); ++i) {
	delete i->second;
    }
}
